@@ -31,7 +31,6 @@
 
 #include <QDebug>
 #include <QtEndian>
-#include <QFile>
 #include <QRegularExpression>
 #include <QString>
 #include <QStringList>
@@ -112,6 +111,7 @@ static void setupPreregisteredOsFlavors() {
     registerOsFlavor(Abi::WindowsMsvc2015Flavor, "msvc2015", {Abi::OS::WindowsOS});
     registerOsFlavor(Abi::WindowsMsvc2017Flavor, "msvc2017", {Abi::OS::WindowsOS});
     registerOsFlavor(Abi::WindowsMsvc2019Flavor, "msvc2019", {Abi::OS::WindowsOS});
+    registerOsFlavor(Abi::WindowsMsvc2022Flavor, "msvc2022", {Abi::OS::WindowsOS});
     registerOsFlavor(Abi::WindowsMSysFlavor, "msys", {Abi::OS::WindowsOS});
     registerOsFlavor(Abi::WindowsCEFlavor, "ce", {Abi::OS::WindowsOS});
     registerOsFlavor(Abi::VxWorksFlavor, "vxworks", {Abi::OS::VxWorks});
@@ -240,6 +240,10 @@ static Abis parseCoffHeader(const QByteArray &data)
         arch = Abi::ArmArchitecture;
         width = 32;
         break;
+    case 0xaa64: // ARM64
+        arch = Abi::ArmArchitecture;
+        width = 64;
+        break;
     case 0x8664: // x86_64
         arch = Abi::X86Architecture;
         width = 64;
@@ -282,7 +286,9 @@ static Abis parseCoffHeader(const QByteArray &data)
             flavor = Abi::WindowsMsvc2013Flavor;
             break;
         case 14:
-            if (minorLinker >= quint8(20))
+            if (minorLinker >= quint8(30))
+                flavor = Abi::WindowsMsvc2022Flavor;
+            else if (minorLinker >= quint8(20))
                 flavor = Abi::WindowsMsvc2019Flavor;
             else if (minorLinker >= quint8(10))
                 flavor = Abi::WindowsMsvc2017Flavor;
@@ -292,6 +298,9 @@ static Abis parseCoffHeader(const QByteArray &data)
         case 15:
             flavor = Abi::WindowsMsvc2019Flavor;
             break;
+        case 16:
+            flavor = Abi::WindowsMsvc2022Flavor;
+            break;
         default: // Keep unknown flavor
             if (minorLinker != 0)
                 flavor = Abi::WindowsMSysFlavor; // MSVC seems to avoid using minor numbers
@@ -687,9 +696,9 @@ bool Abi::operator == (const Abi &other) const
 
 static bool compatibleMSVCFlavors(const Abi::OSFlavor &left, const Abi ::OSFlavor &right)
 {
-    // MSVC 2019, 2017 and 2015 are compatible
-    return left >= Abi::WindowsMsvc2015Flavor && left <= Abi::WindowsMsvc2019Flavor
-           && right >= Abi::WindowsMsvc2015Flavor && right <= Abi::WindowsMsvc2019Flavor;
+    // MSVC 2022, 2019, 2017 and 2015 are compatible
+    return left >= Abi::WindowsMsvc2015Flavor && left <= Abi::WindowsMsvc2022Flavor
+           && right >= Abi::WindowsMsvc2015Flavor && right <= Abi::WindowsMsvc2022Flavor;
 }
 
 bool Abi::isCompatibleWith(const Abi &other) const
@@ -1086,6 +1095,8 @@ bool Abi::osSupportsFlavor(const Abi::OS &os, const Abi::OSFlavor &flavor)
 
 Abi::OSFlavor Abi::flavorForMsvcVersion(int version)
 {
+    if (version >= 1930)
+        return WindowsMsvc2022Flavor;
     if (version >= 1920)
         return WindowsMsvc2019Flavor;
     if (version >= 1910)
@@ -1157,14 +1168,7 @@ Abis Abi::abisOfBinary(const Utils::FilePath &path)
     if (path.isEmpty())
         return tmp;
 
-    QFile f(path.toString());
-    if (!f.exists())
-        return tmp;
-
-    if (!f.open(QFile::ReadOnly))
-        return tmp;
-
-    QByteArray data = f.read(1024);
+    QByteArray data = path.fileContents(1024);
     if (data.size() >= 67
             && getUint8(data, 0) == '!' && getUint8(data, 1) == '<' && getUint8(data, 2) == 'a'
             && getUint8(data, 3) == 'r' && getUint8(data, 4) == 'c' && getUint8(data, 5) == 'h'
@@ -1197,13 +1201,11 @@ Abis Abi::abisOfBinary(const Utils::FilePath &path)
                 break;
 
             offset += (offset % 2); // ar is 2 byte aligned
-            f.seek(offset);
-            data = f.read(1024);
+            data = path.fileContents(1024, offset);
         }
     } else {
         tmp = abiOf(data);
     }
-    f.close();
 
     // Remove duplicates:
     Abis result;
@@ -1409,8 +1411,10 @@ void ProjectExplorer::ProjectExplorerPlugin::testAbiOfBinary()
     QFETCH(QStringList, abis);
 
     const Utils::FilePath fp = Utils::FilePath::fromString(file);
-    if (!fp.exists())
+    const QString dataTag = QString::fromLocal8Bit(QTest::currentDataTag());
+    if (!fp.exists() && dataTag != "no file" && dataTag != "non existing file")
         QSKIP("binary file not present");
+
     const Abis result = Abi::abisOfBinary(fp);
     QCOMPARE(result.count(), abis.count());
     for (int i = 0; i < abis.count(); ++i)
